using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.SqlServer.Management.UI.VSIntegration;
using Microsoft.SqlServer.Management.UI.VSIntegration.Editors;
using Microsoft.SqlServer.Management.Smo.RegSvrEnum;
using Microsoft.SqlServer.Management.Common;
using Microsoft.SqlServer.Management.Smo;
using System.Reflection;
using System.Windows;
using EnvDTE80;
using EnvDTE;
using Microsoft.SqlServer.Management.UI.VSIntegration.ObjectExplorer;
using System.Threading;
using System.Text.RegularExpressions;

namespace DatabaseObjectSearcher
{
    public class SearcherController
    {
        static SearcherController currentInstance = new SearcherController();

        private DTE2 _application;
        private UIConnectionInfo currentUIConnection;
        private Dictionary<string, DatabaseObjectSearcher> searcherDictionary = new Dictionary<string, DatabaseObjectSearcher>();
        private SqlConnectionInfo currentConnection;
        private Window toolWindow;

        public static SearcherController Current
        {
            get { return currentInstance; }
        }

        public DatabaseObjectSearcher GetSearcher()
        {
            SqlConnectionInfo connectionInfo = GetActiveWindowConnection();
            if (connectionInfo != null)
            {
                lock (searcherDictionary)
                {
                    if (searcherDictionary.ContainsKey(connectionInfo.ConnectionString))
                    {
                        return searcherDictionary[connectionInfo.ConnectionString];
                    }
                    else
                    {
                        DatabaseObjectSearcher searcher = new DatabaseObjectSearcher(connectionInfo);
                        searcherDictionary.Add(connectionInfo.ConnectionString, searcher);
                        ThreadPool.QueueUserWorkItem(new WaitCallback(searcher.BuildDBObjectDictionary));
                        return searcher;
                    }
                }
            }
            return null;
        }

        private SqlConnectionInfo GetActiveWindowConnection()
        {
            SqlConnectionInfo info = null;
            try
            {
                UIConnectionInfo connInfo = null;
                if (ServiceCache.ScriptFactory.CurrentlyActiveWndConnectionInfo != null)
                {
                    connInfo = ServiceCache.ScriptFactory.CurrentlyActiveWndConnectionInfo.UIConnectionInfo;
                }
                if (connInfo != null)
                {
                    if (connInfo == currentUIConnection)
                    {
                        return currentConnection;
                    }
                    else
                    {
                        info = CreateSqlConnectionInfo(connInfo);
                        currentConnection = info; 
                        currentUIConnection = connInfo;
                    }
                }
                if (info == null)
                {
                    // attempt to get connection info from node in tree view
                    INodeInformation[] nodes = GetObjectExplorerSelectedNodes();
                    if (nodes.Length > 0)
                    {
                        info = nodes[0].Connection as SqlConnectionInfo;
                    }
                }
                return info;
            }
            catch (NullReferenceException)
            {
                return null;
            }
        }

        private INodeInformation[] GetObjectExplorerSelectedNodes()
        {
            IObjectExplorerService objExplorer = ServiceCache.GetObjectExplorer();
            int arraySize;
            INodeInformation[] nodes;
            objExplorer.GetSelectedNodes(out arraySize, out nodes);
            return nodes;
        }

        private SqlConnectionInfo CreateSqlConnectionInfo(UIConnectionInfo connectionInfo)
        {
            SqlConnectionInfo sqlConnInfo = new SqlConnectionInfo();
            sqlConnInfo.ServerName = connectionInfo.ServerName;
            sqlConnInfo.UserName = connectionInfo.UserName;
            if (string.IsNullOrEmpty(connectionInfo.Password)) // TODO - this is a bit of a hack
            {
                sqlConnInfo.UseIntegratedSecurity = true;
            }
            else
            {
                sqlConnInfo.Password = connectionInfo.Password;
            }

            return sqlConnInfo;
        }

        public void SelectSMOObject(NamedSmoObject objectToSelect, SqlConnectionInfo connection)
        {
            ObjectExplorerManager manager = new ObjectExplorerManager();
            manager.SelectSMOObjectInObjectExplorer(objectToSelect, connection); 
            
        }

        public Window SearchWindow
        {
            get { return toolWindow; }
        }
        
        public void CreateAddinWindow(DTE2 application, AddIn addinInstance)
        {
            Assembly asm = Assembly.Load("DatabaseObjectSearcherUI");
            Guid id = new Guid("4c410c93-d66b-495a-9de2-99d5bde4a3b9"); // this guid doesn't seem to matter?
            toolWindow = CreateToolWindow("DatabaseObjectSearcherUI.SearchToolWindow", asm.Location, id, application, addinInstance);
            if (_application == null)
            {
                _application = application;
            }
        }

        private Window CreateToolWindow(string typeName, string assemblyLocation, Guid uiTypeGuid, DTE2 applicationObject, AddIn addinInstance)
        {
            Windows2 win2 = applicationObject.Windows as Windows2;
            if (win2 != null)
            {
                object controlObject = null;
                Assembly asm = Assembly.GetExecutingAssembly();
                Window toolWindow = win2.CreateToolWindow2(addinInstance, assemblyLocation, typeName, "DB Object QuickFind", "{" + uiTypeGuid.ToString() + "}", ref controlObject);
                toolWindow.Visible = true;
                return toolWindow;
            }
            return null;
        }
    }
}
